unit VersInfo;
{
  File Version Information Component

  Extracts version information from files: .EXE, .DLL, .DPL's, etc.
  Gets fixed version data as well as language/codepage dependent information.
  New data can be loaded by simply changing the FileName property.

  For Delphi32 only - see WIN32.HLP
  Copyright (c) 1997, by Philip Stevenson
}

interface

 uses Windows, SysUtils, Dialogs, Classes;

const
  Keys = 10;

type
  EVersInfoError = class(Exception);

  {Version Information class}
  TVersInfo = class(TComponent)
  private
    FFilename: string;
    BufSize: integer;
    VIBuffer: pointer;
    FixedFileInfoBuf: PVSFixedFileInfo;    { Fixed File Information block }
    HaveFixedData: boolean;
    CodeStr: string;
    FLanguage: word;
    FCharSet: word;
    FVersionKeys: TStrings;
    FVersionInfo: TStrings;
    FIncludeTitles: boolean;
    procedure ExtractVersionData;
    procedure SetFileName(Value : string);
    function ErrFmt(num: integer): string;
    function LanguageDependentInfoLoaded: boolean;
    function FixedDataLoaded: boolean;
    function GetVersion(index: integer): string;
    function GetFileAttributes(index: integer): Longint;
    function GetLangCharSet(index: integer): word;
    function GetLanguageName: string;
    function GetVersionInfoString(index: integer): string;
    function GetVersionKeys: TStrings;
    function GetVersionInfoStrings: TStrings;
    procedure SetIncludeTitles(Value : boolean);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    class function AddSpace(const S: string): string;

    { Fixed Version-info (partial) }
    property ModuleVersionNumber: string index 3 read GetVersion;
    property ModuleProductVersion: string index 5 read GetVersion;
    property ModuleFlagsMask: Longint index 7 read GetFileAttributes;
    property ModuleFlags: Longint index 8 read GetFileAttributes;
    property ModuleOS: Longint index 9 read GetFileAttributes;
    property ModuleType: Longint index 10 read GetFileAttributes;

    { Lang/Codeset dependent version-information }
    property LanguageID: word index 1 read GetLangCharSet;
    property CharSet:    word index 2 read GetLangCharSet;
    property LanguageName: string read GetLanguageName;
    property CompanyName:      string index 1 read GetVersionInfoString;
    property Description:      string index 2 read GetVersionInfoString;
    property FileVersion:      string index 3 read GetVersionInfoString;
    property InternalName:     string index 4 read GetVersionInfoString;
    property Copyright:        string index 5 read GetVersionInfoString;
    property Trademarks:       string index 6 read GetVersionInfoString;
    property OriginalFileName: string index 7 read GetVersionInfoString;
    property ProductName:      string index 8 read GetVersionInfoString;
    property ProductVersion:   string index 9 read GetVersionInfoString;
    property Comments:         string index 10 read GetVersionInfoString;
    property VersionKeys: TStrings read GetVersionKeys;
    property VersionInfo: TStrings read GetVersionInfoStrings;
  published
    property FileName: string read FFileName write SetFileName;
    property IncludeTitles: boolean read FIncludeTitles write SetIncludeTitles;

  end;

  procedure Register;

  implementation

  const
    VersInfoKey: array[1..Keys] of string[20] =
     ('CompanyName',      'FileDescription',
      'FileVersion',      'InternalName',
      'LegalCopyright',   'Trademarks',
      'OriginalFileName', 'ProductName',
      'ProductVersion',   'Comments');

    ErrMsgs: array[1..5] of string[60] =
     ('No file specified',
      'File %s not found or not accessible',
      'File %s has no version information',
      'Cannot read version information for file %s',
      'Cannot read translation table for file %s');


class function TVersInfo.AddSpace(const S: string): string;
{-Add spaces before capital letters, except for first}
var
  ix: integer;
begin
  Result := '';
  for ix := 1 to Length(S) do
    if (S[ix] in ['A'..'Z']) and (ix > 1) then
      Result := Result + ' ' + S[ix]
    else
      Result := Result + S[ix]
end;

procedure Register;
begin
  RegisterComponents('Samples', [TVersInfo]);
end;

constructor TVersInfo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FVersionKeys := TStringList.Create;
  FVersionInfo := TStringList.Create;
end;

destructor TVersInfo.Destroy;
begin
  if Assigned(VIBuffer) then
    FreeMem(VIBuffer, BufSize);
  FVersionKeys.Free;
  FVersionInfo.Free;
  inherited Destroy;
end;

function TVersInfo.ErrFmt(num: integer): string;
begin
  Result := Format(ErrMsgs[num], [ExtractFileName(FFileName)])
end;

{-Method to find and get the data}
procedure TVersInfo.ExtractVersionData;
var
  VHandle: Cardinal;

begin
  HaveFixedData := False;
  CodeStr := '';
  if Assigned(VIBuffer) then
  begin
    FreeMem(VIBuffer, BufSize);
    VIBuffer := Nil;
  end;
  {Find file and get a buffer}
  BufSize := GetFileVersionInfoSize(PChar(FFileName), VHandle);
  if BufSize = 0 then
  begin
    if FileExists(FFileName) then
      raise EVersInfoError.Create(ErrFmt(3));
    raise EVersInfoError.Create(ErrFmt(2));
  end;
  GetMem(VIBuffer, BufSize);

  {Fill the buffer}
  if not GetFileVersionInfo(PChar(FFileName), VHandle, BufSize, VIBuffer) then
    raise EVersInfoError.Create(ErrFmt(4));
end;

procedure TVersInfo.SetFileName(Value : String);
begin
  FFileName := Value;
  if not(csDesigning in ComponentState) then
    ExtractVersionData;
end;

{-Load fixed file version data}
function TVersInfo.FixedDataLoaded: boolean;
var
  Len: Cardinal;
begin
  Result := False;
  if FFileName = '' then
    raise EVersInfoError.Create(ErrMsgs[1]);
  if not HaveFixedData then
    if VerQueryValue(VIBuffer, '\', Pointer(FixedFileInfoBuf), Len) then
    begin
      if Len < SizeOf(TVSFixedFileInfo) then  {not much or none}
        exit;
      HaveFixedData := True;
    end;
  Result := HaveFixedData;
end;

{-Get the version numbers from fixed data}
function TVersInfo.GetVersion(index: integer): string;

  function VersionNumber(const MS, LS: Longint): string;
  begin
    Result := Format('%d.%d.%d.%d',
      [HIWORD(MS), LOWORD(MS), HIWORD(LS), LOWORD(LS)]);
  end;

begin
  Result := '';
  if FixedDataLoaded then
    with FixedFileInfoBuf^ do
    case index of
      3: Result := VersionNumber(dwFileVersionMS, dwFileVersionLS);
      5: Result := VersionNumber(dwProductVersionMS, dwProductVersionLS);
    end;
end;

{-Get some of the file attributes }
function TVersInfo.GetFileAttributes(index: Integer): Longint;
begin
  Result := 0;
  if FixedDataLoaded then
    with FixedFileInfoBuf^ do
    case index of
      7: Result := dwFileFlagsMask;
      8: Result := dwFileFlags;
      9: Result := dwFileOS;
     10: Result := dwFileType;
    end;
end;


{-Get LanguageID/Codeset}
function TVersInfo.LanguageDependentInfoLoaded: boolean;
var
  vibptr: ^Longint; {pointer to buffer of vers info values }
  rb: LongBool;
  Len: Cardinal;

begin
  if FFileName = '' then
    raise EVersInfoError.Create(ErrMsgs[1]);
  if CodeStr = '' then
  begin
    rb := VerQueryValue(VIBuffer, '\VarFileInfo\Translation',
      Pointer(vibptr), Len);
    if rb and (Len >= 4) then    { Translation table exists}
    begin
      { copy the 4 byte translation entry for the first }
      FLanguage := LOWORD(vibptr^);
      FCharSet := HIWORD(vibptr^);
      {  note embedded hex codes must have leading zeros}
      CodeStr := Format('\StringFileInfo\%.4x%.4x\', [FLanguage, FCharSet]);
    end
    else
      raise EVersInfoError.Create(ErrFmt(5));
  end;
  Result := CodeStr <> '';
end;

{-Return Language ID or codepage ID}
function TVersInfo.GetLangCharSet(index: Integer): word;
begin
  Result := 0;
  if LanguageDependentInfoLoaded then
    case index of
      1: Result := FLanguage;
      2: Result := FCharSet;
    end;
end;

{-Return name of the language used}
function TVersInfo.GetLanguageName: string;
var
  buffer: array[0..80] of char;
begin
  Result := '';
  if LanguageDependentInfoLoaded then
  begin
    VerLanguageName(FLanguage, buffer, 80); { Get Language description }
    Result := string(buffer);
  end;
end;

{-Return one item of version info (e.g. CompanyName) }
function TVersInfo.GetVersionInfoString(index: Integer): string;
var
  P: PChar;
  Len: Cardinal;
  TranslateStr: string;
begin
  Result := '';
  if LanguageDependentInfoLoaded then
  begin
    TranslateStr := CodeStr + VersInfoKey[index]; {item to get}
    if VerQueryValue(VIBuffer, PChar(TranslateStr), Pointer(P), Len) then
      Result := P;
  end;
end;

{-Return all version information strings}
function TVersInfo.GetVersionInfoStrings: TStrings;
var
  ix: integer;
begin
  Result := FVersionInfo;
  FVersionInfo.Clear;
  if LanguageDependentInfoLoaded then
  begin
    if IncludeTitles then
      FVersionInfo.Add('Version Description');
    for ix := 1 to Keys do
      FVersionInfo.Add(GetVersionInfoString(ix));
  end;
end;

{-Return the predefined version strings in this component}
function TVersInfo.GetVersionKeys: TStrings;
var
  ix: integer;
begin
  Result := FVersionKeys;
  if FVersionKeys.Count > 0 then {is loaded}
    exit;
  if IncludeTitles then
    FVersionKeys.Add('Version Key');
  for ix := 1 to Keys do
    FVersionKeys.Add(AddSpace(VersInfoKey[ix]));
end;

{-Change IncludeTitles}
procedure TVersInfo.SetIncludeTitles(Value : boolean);
begin
  FIncludeTitles := Value;
  FVersionKeys.Clear;
  FVersionInfo.Clear;
end;

end.
